////////////////////////////////////////////////////////////////////////
// Temperature_I2C.h  - Temperature Module
//
// Temperature_I2C encapsulates the logic to initialize and drive the I2C8
// bus in order to read the Digital Microtrak 3.1 board temperature from a
// TMP102 chip.
//
//
////////////////////////////////////////////////////////////////////////
#include <stdint.h>
#include <stdbool.h>

//-------------------
// MWARE Header files
//-------------------
#include "inc/hw_memmap.h"
#include "inc/hw_types.h"
#include "inc/hw_sysctl.h"
#include "inc/hw_i2c.h"

#include "driverlib/pin_map.h"
#include "driverlib/i2c.h"
#include "driverlib/gpio.h"
#include "driverlib/sysctl.h"
#include "driverlib/rom_map.h"

//-------------------------
// Application Header Files
//-------------------------
#include "Temperature_I2C.h"
#include "Common.h"
#include "Parameters.h"


//*****************************************************************************
// Set the address for slave module. This is a 7-bit address sent in the
// following format:
//                      [A6:A5:A4:A3:A2:A1:A0:RS]
//
// A zero in the "RS" position of the first byte means that the master
// transmits (sends) data to the selected slave, and a one in this position
// means that the master receives data from the slave.
//
// The slave address for the TMP102 is 1001000
//*****************************************************************************
#define TEMPERATURE_I2C_BASE              I2C8_BASE

#define TEMPERATURE_SLAVE_ADDRESS         0x48				// 0100 1000

void Temperature_I2C_Initialize(void)
{
	// The I2C0 peripheral must be enabled before use.
	MAP_SysCtlPeripheralEnable(SYSCTL_PERIPH_I2C8);

  // Enable and initialize the I2C8 master module.  Use the system clock for
  // the module.  The last parameter sets the I2C data transfer rate.
  // If false the data rate is set to 100kbps and if true the data rate will
  // be set to 400kbps.  We will use a data rate of 400kbps.
  MAP_I2CMasterInitExpClk(TEMPERATURE_I2C_BASE, PRM_GetSystemClockFrequency(), true);
}

//*************************************************************************************
// float32 Temperature_I2C_Read(void)
//
// This function uses the I2C to read the current temperature in the register.
// on a TMP102.
//
//*************************************************************************************
float32 Temperature_I2C_Read(void)
{
	uint32_t low_byte;
	uint32_t high_byte;

  float32 tempFloat;
  sint16 tempCnts;

  //Setup to do a read to chip
  MAP_I2CMasterSlaveAddrSet(TEMPERATURE_I2C_BASE, TEMPERATURE_SLAVE_ADDRESS, true);

  //Make sure bus is free
  while( MAP_I2CMasterBusBusy(TEMPERATURE_I2C_BASE) || MAP_I2CMasterBusy(TEMPERATURE_I2C_BASE) ){}

  // Initiate Read:
  MAP_I2CMasterControl(TEMPERATURE_I2C_BASE, I2C_MASTER_CMD_BURST_RECEIVE_START);

  //Delay until transmission starts
  while( !MAP_I2CMasterBusy(TEMPERATURE_I2C_BASE) ){}

  //Delay until transmission completes
  while( MAP_I2CMasterBusy(TEMPERATURE_I2C_BASE) ){}

  //Check to see if the device responded:
  if( MAP_I2CMasterErr(TEMPERATURE_I2C_BASE) != I2C_MASTER_ERR_NONE )
  {
    return ERROR_MEASUREMENT;
  }

  //Retrieve the data byte
  high_byte = MAP_I2CMasterDataGet(TEMPERATURE_I2C_BASE);

  // Initiate Read:
  MAP_I2CMasterControl(TEMPERATURE_I2C_BASE, I2C_MASTER_CMD_BURST_RECEIVE_FINISH);

  //Delay until transmission starts
  while( !MAP_I2CMasterBusy(TEMPERATURE_I2C_BASE) ){}

  //Delay until transmission completes
  while( MAP_I2CMasterBusy(TEMPERATURE_I2C_BASE) ){}

  //Delay until transmission completes
  //while( MAP_I2CMasterBusBusy(TEMPERATURE_I2C_BASE) || MAP_I2CMasterBusy(TEMPERATURE_I2C_BASE) ){};

  //Check to see if the device responded:
  if( MAP_I2CMasterErr(TEMPERATURE_I2C_BASE) != I2C_MASTER_ERR_NONE )
  {
    return ERROR_MEASUREMENT;
  }

  //Retrieve the data byte
  low_byte = MAP_I2CMasterDataGet(TEMPERATURE_I2C_BASE);

  //Each read should only return 8 bits
  low_byte &= 0x000000FF;
  high_byte &= 0x000000FF;

  tempCnts = (sint16)( (high_byte << 8) | low_byte );
  tempCnts = tempCnts >> 4;    //Should sign extend

  tempFloat = (float32)tempCnts * 0.0625; //Compute temperature

	return tempFloat;
}
